<?php

/**
 * this class handles the file download for file pages.
 * it uses the pear downloader for that purpose.
 *
 */
class tx_filepage {
	
	/**
	 * returns a filename that is based on the given $caption and has the same
	 * file extension as the given $filename.
	 *
	 * @param string $caption arbitrary text (utf-8)
	 * @param string $filename
	 * @return string (ascii)
	 */
	static protected function get_filename_from_caption($caption, $filename) {
		$file_func_obj= t3lib_div::makeInstance('t3lib_basicFileFunctions');
		$clean_filename= $file_func_obj->cleanFileName($filename); // ascii
		$pinfo= pathinfo($clean_filename);
		$ext= strtolower($pinfo['extension']);
		if (empty($ext)) {
			$tmp= $caption;
		} else {
			$tmp= $caption.'.'.$ext;
		}
		
		return $file_func_obj->cleanFileName($tmp, 'utf-8');
	}
	
	/**
	 * clear (discard) all output buffers recursively.
	 *
	 */
	static protected function clear_all_ob() {
	   while (@ob_end_clean());
	}
	
	/**
	 * output post processing
	 * 
	 * The download is served here.
	 *
	 * @param unknown_type $content
	 * @param tslib_fe $pObj
	 */
	function parseFE(&$content, $pObj) {
		if ($content['pObj']->page['doktype'] != tx_filepage_config::get('page_type_id')) {
			return;
		}
		
		$upload= $content['pObj']->page['tx_filepage_upload']; // comma-separated list of file names
		$folder= tx_filepage_config::get('uploadfolder');
		$caption= $content['pObj']->page['title'];
		if (strlen($caption) <= 0) {
			$caption= $content['pObj']->page['nav_title'];
		}
		
		// if page is disabled / deleted, don't deliver it to user!
		if ($content['pObj']->pageNotFound != 0) {
			return;
		}
		
		// does page contain upload?
		if (strlen($upload) > 0) {
			// yes, send upload to user
			
			$uploaded_files= explode(',', $upload);
			
			// get last file. this is the active file.
			$active_file= array_pop($uploaded_files);
			 
			// check existence
			$abs_path= PATH_site.$folder.'/'.$active_file;
			
			if (!file_exists($abs_path)) {
				$content['pObj']->content= $pObj->sL('LLL:EXT:filepage/locallang.xml:output.file_not_found');
			} elseif (!is_readable($abs_path)) {
				$content['pObj']->content= $pObj->sL('LLL:EXT:filepage/locallang.xml:output.file_not_readable');
			} else {
				$output_filename= self::get_filename_from_caption($caption, $active_file); // ascii
				
				$orig_include_path= ini_get('include_path');
				if (tx_filepage_config::get('use_builtin_pear_lib') == 1) {
					ini_set('include_path', dirname(__FILE__).'/res/pear');
				} elseif (strlen(tx_filepage_config::get('use_pear_lib')) > 0) {
					ini_set('include_path', tx_filepage_config::get('use_pear_lib'));
				}
		
				// include pear library
				require_once('HTTP/Download.php');
	
				ini_set('max_execution_time', 3600);
				ini_set('max_input_time', 180);
				ini_set('memory_limit', tx_filepage_config::get('max_memory_limit', -1, -1));
				ini_set('ignore_user_abort', 0);
	
				// discard all output buffers
				self::clear_all_ob();
					
				// http://msdn2.microsoft.com/en-us/library/ms775148.aspx
				// remove http headers that may have been sent by previously used output compression  
				if (function_exists('header_remove')) {
					header_remove('Content-Encoding');
					header_remove('Vary');
				} else {
					header('Content-Encoding: ', true); // remove content-encoding header
					//header('Transfer-Encoding: ', true); // remove transfer-encoding header
					header('Vary: ', true); // remove vary header
				}
				//header('Content-Description: '.trim(htmlentities($caption)));
				//header(sprintf('Content-Type: %s', $mimetype));
	
				$dl= new HTTP_Download();
				$dl->setFile($abs_path);
				$dl->setGzip(false); // do not use compression at the moment. got problems.
				$dl->setCache(true);
				$dl->setContentDisposition(HTTP_DOWNLOAD_ATTACHMENT, trim(htmlspecialchars($output_filename)));
				$dl->guessContentType();
				$dl->send();
				
				ini_set('include_path', $orig_include_path);
				exit;
			}
		}
	}
	
	function set_title_wizard($params, &$tceform) {
		/*
		 * $params['itemName'] // form element name p.e. "data[tt_content][10053][tx_hmtfw_mtag_content]"
		 * $params['formName'] // form name p.e. "editform"
		 * $params['table'] // p.e. "tt_content"
		 * $params['uid'] // p.e. 100050
		 * $params['pid'] // parent id of current element
		 * $params['field'] // db field p.e. "tx_hmtfw_mtag_content"
		 * $params['md5ID'] // p.e. "IDf0767c9806"
		 * $params['returnUrl'] // p.e. "/t3/typo3/alt_doc.php?&returnUrl=%2Ft3%2Ftypo3%2Fsysext%2Fcms%2Flayout%2Fdb_layout.php%3Fid%3D13908&edit[tt_content][10053]=edit"
		 * &$params['item'] // reference to html representation of item p.e. "<textarea name="data[tt_content][10053][tx_hmtfw_mtag_content]" style="width:460px;" cols="48" class="formField1" rows="20" wrap="virtual" onchange="TBE_EDITOR.fieldChanged('tt_content','10053','tx_hmtfw_mtag_content','data[tt_content][10053][tx_hmtfw_mtag_content]');">[2: Noch ein M-Tag-Inhalt]</textarea>"
		 * $params['icon'] // p.e. "M-Tag-Vorschau"
		 * $params['iTitle'] // p.e. "M-Tag-Vorschau"
		 * $params['wConf'] // wizard configuration
		 * $params['row'] // db row of current element
		 * $params['fieldChangeFunc'] // array of js statements. call these after modifying the item's value by javascript or the value will not be saved.
		 * 
		 * &$tceform // object(t3lib_TCEforms)
		 */
		
		if (!tx_filepage_config::get('enable_set_title_button', null, false)) {
			// "set page title" button is disabled
			return '';
		}
		
		$base_input_name= $params['itemName']; // e.g. data[pages][14261][tx_filepage_upload]
		$upload_input_name= $base_input_name; // e.g. data_files[pages][14261][tx_filepage_upload]
		$upload_input_name= str_replace('data[', 'data_files[', $upload_input_name);
		$list_input_name= $base_input_name.'_list'; // e.g. data[pages][14261][tx_filepage_upload]_list
		$title_input_name= str_replace('tx_filepage_upload', 'title', $base_input_name).'_hr'; // e.g. data[pages][14261][title]_hr
		$selector_name= 'settitleselector'.$base_input_name;
		
		$js=
			'try {'.
			'var title= document.getElementsByName(\''.$title_input_name.'\')[0]; '.
			'var upload= document.getElementsByName(\''.$upload_input_name.'\')[0]; '.
			'var list= document.getElementsByName(\''.$list_input_name.'\')[0]; '.
			'var selector= document.getElementsByName(\''.$selector_name.'\')[0]; '.
			'var path= \'\'; 
			var nice= true;
			try { nice= selector.checked; } catch (e) {}
			try { path= upload.value; } catch (e) {}
			if (path.length == 0) {
				try { path= list[list.length - 1].value; } catch (e) {}
			}
			if (path.length > 0) {
				var name= \'\';
 				var idxa= path.lastIndexOf(\'/\') + 1;
  				var idxc= path.lastIndexOf(\'\\\\\') + 1;
  				idxa= (idxc > idxa) ? idxc : idxa;
  				var idxb= path.length;
  				
  				if (idxa < idxb) {
  					name= path.substring(idxa, idxb);
  				}
  				
  				idxb= name.lastIndexOf(\'.\');
  				if (idxb > 0) {
  					name= name.substring(0, idxb);
  				}
  				
  				if (name.length > 0) {
  					if (nice) {
	  					var pos= name.indexOf(\'_\');
						while (pos >= 0) {
							name= name.replace(/_+/, \' \');
							pos= name.indexOf(\'_\');
						}
					
						var ar= name.split(/\\s+/);
						var caption= \'\';
						for (var i= 0; i < ar.length; i++ ) {
							if (caption.length > 0) {
								caption= caption + \' \';
							}
							if (ar[i] === ar[i].toUpperCase()) {
								caption= caption + ar[i];
							} else {
								caption= caption + ar[i].substring(0, 1).toUpperCase();
								caption= caption + ar[i].substring(1, ar[i].length).toLowerCase();
							}
						}
					} else {
						caption= name;
					}
					
					title.value= caption;
					title.onchange();
  				}
			} 
			'.
			'} catch(e) { }';
		
		$link_name= $tceform->sL('LLL:EXT:filepage/locallang.xml:button_label.set_title');
		$link= '<div style="margin-top: 12px; display: inline-block; border: 1px solid #666666; background-color: white; padding: 5px;"><a onclick="'.str_replace(array(chr(13).chr(10), chr(13), chr(10), ), ' ', $js).' return false;" href="#">'.$link_name.'</a></div>';
		$selector_label_nice= $tceform->sL('LLL:EXT:filepage/locallang.xml:button_mode_label.nice');
		$selector_label_raw= $tceform->sL('LLL:EXT:filepage/locallang.xml:button_mode_label.raw');
		$selector= 
			'<div style="margin-left: 10px; display: inline-block"><input type="Radio" name="'.$selector_name.'" value="nice" checked>&nbsp;'.$selector_label_nice.
			'&nbsp;&nbsp;&nbsp;<input type="Radio" name="'.$selector_name.'" value="raw">&nbsp;'.$selector_label_raw.'</div>';
		
		$params['item'].= '<div>'.$link.$selector.'</div>';
		return '';
	}
	
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/filepage/class.tx_filepage.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/filepage/class.tx_filepage.php']);
}

?>