<?php

/**
 * this class is used by the filepage extension to read its configuration.
 *
 */
class tx_filepage_config {
	
	/**
	 * the extension key
	 *
	 */
	const EXT= 'filepage';
	
	static protected $config_loaded= false;
	static protected $params;
	static protected $params_set;
	
	static protected function load_config() {
		if (self::$config_loaded) {
			return;
		}
		
		$config= $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf'][self::EXT];
		if (strlen($config) > 0) {
			$config= unserialize($config);
			if ($config !== false) {
				foreach ($config as $key => $value) {
					self::$params[$key]= $value;
					self::$params_set[$key]= true;
				}
			}
		}
		self::$config_loaded= true;
	}
	
	/**
	 * returns true if the given parameter has been set. it returns true, even if the value has been set to NULL!
	 *
	 * @param string $param
	 * @return bool
	 */
	static public function is_set($param) {
		self::load_config();
		return self::$params_set[$param] == true;
	}
	
	static public function get($param, $default_if_value_is_nothing= null, $default_if_value_is_undefined= null) {
		self::load_config();
		if (self::is_set($param)) {
			$value= self::$params[$param];
			if ($default_if_value_is_nothing !== null) {
				return ifnothing($value, $default_if_value_is_nothing);
			} else {
				return $value;
			}
		} elseif ($default_if_value_is_undefined !== null) {
			return $default_if_value_is_undefined;
		} else {
			self::send_error('param undefined: "'.$param.'"');
		}
	}
	
	static public function set($param, $value) {
		self::load_config();
		self::$params[$param]= $value;
		self::$params_set[$param]= true;
	}
	
	static public function set_if_undefined($param, $value) {
		if (self::is_set($param)) {
			return;
		}
		self::set($param, $value);
	}
	
	static public function un_set($param) {
		self::load_config();
		unset(self::$params[$param]);
		self::$params_set[$param]= false;
	}
	
	/**
	 * writes a short generic message to the user, writes $msg to the php error log and terminates.
	 *
	 * @param string $msg
	 */
	static public function send_error($msg) {
		echo '<br><br>application error.';
		self::send_warning($msg);
		exit;
	}
	
	/**
	 * writes $msg to the php error log.
	 *
	 * @param string $msg
	 */
	static public function send_warning($msg) {
		$caller_stack= '';
		try {
			throw new Exception();
		} catch (Exception $e) {
			$caller_stack.= $msg."\n";
			$caller_stack.= $e->getTraceAsString();
		}
		if (strlen($caller_stack) <= 0) {
			$caller_stack= __CLASS__.'::'.__METHOD__.'('.$msg.')';
		}
		error_log($caller_stack);
	}
	
	static public function dump() {
		$data= self::$params;
		ksort($data);
		echo '<hr><pre>';
		var_dump($data);
		echo '</pre><hr>';
	}
	
}

// this is a static class. therefore we have no xclass clause here. 

?>