<?php

/**
 * static helper class that implements the upload folder security check and the file page view for the backend.
 *
 */
class tx_filepage_helper {
	
	static public function security_check() {
		// try to create a test file in the upload folder or use an existing one
		$test_file_name= '_filepage_accesscheck.txt';
		$test_file_path= PATH_site.tx_filepage_config::get('uploadfolder').'/'.$test_file_name;
		if (!file_exists($test_file_path)) {
			// try to create file
			if (!t3lib_div::writeFile($test_file_path, 'I am a cool test file.')) {
				// write failed
				
				// fallback: try to use an existing file
				$file_ar= t3lib_div::getFilesInDir(PATH_site.tx_filepage_config::get('uploadfolder'), '', false, '', '.htaccess,.htpasswd,.,..');
				$test_file_name= array_pop($file_ar);
				if (strlen($test_file_name) == 0) {
					$test_file_path= '';
				} else {
					$test_file_path= PATH_site.tx_filepage_config::get('uploadfolder').'/'.$test_file_name;
				}
			}
		}

		if (strlen($test_file_name) == 0) {
			return 'security_check.could_not_check';
		}
		
		// test access
		$test_url= t3lib_div::getIndpEnv('TYPO3_SITE_URL').tx_filepage_config::get('uploadfolder').'/'.$test_file_name;
		$access_test_file= @file_get_contents($test_url) !== false; // true, if web access was possible, false otherwise.
		
		if ($access_test_file) {
			return 'security_check.failed';
		}
		
		// access failed. this could be due to a correct protection of the upload folder. but we must first test if we can successfully access an unprotected control file on the webserver.
		$control_file_path= t3lib_div::tempnam('_filepage_control_file_');
		$control_file_path= t3lib_div::fixWindowsFilePath($control_file_path);

		// try to create a temp file
		if (t3lib_div::writeFileToTypo3tempDir($control_file_path, 'I am a super cool control file.') != false) {
			// try to access a web page
			$control_url= t3lib_div::getIndpEnv('TYPO3_SITE_URL').'/typo3/';
		} else {
			$control_file_base_path= str_replace(PATH_site, '', $control_file_path); // typo3temp/some_file_name
			$control_url= t3lib_div::getIndpEnv('TYPO3_SITE_URL').$control_file_base_path;
		}
		
		$access_control_file= @file_get_contents($control_url) !== false; // true, if web access was possible, false otherwise.
		
		// delete temp file
		t3lib_div::unlink_tempfile($control_file_path);
		
		if ($access_control_file) {
			return 'security_check.successful';
		} else {
			return 'security_check.could_not_check';
		}
	}
	
	static public function get_page_view($id, $pageRecord, $backPath) {
		$msg_top= '';
		$msg_bottom= '';
		
		if (tx_filepage_config::get('do_security_check_on_page_module')) {
			$check_result= self::security_check();
			
			$security_msg_title= $GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:'.$check_result.'.title');
			$security_msg_error_type= $GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:'.$check_result.'.error_type');
			$security_msg_body= $GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:'.$check_result.'.text');
			
			$log_error= false;
			
			switch ($security_msg_error_type) {
				case '_error':
					$box_type= t3lib_FlashMessage::ERROR;
					$make_box= true;
					$log_error= true;
					break;
					
				case '_warning':
					$box_type= t3lib_FlashMessage::WARNING;
					$make_box= true;
					$log_error= true;
					break;
					
				case '_info':
					$box_type= t3lib_FlashMessage::INFO;
					$make_box= true;
					break;
					
				case '_notice':
					$box_type= t3lib_FlashMessage::NOTICE;
					$make_box= true;
					break;
					
				case '_plain':
				default:
					$make_box= false;
					break;
			}
			
			if ($log_error) {
				t3lib_div::sysLog($security_msg_title.': '.$security_msg_body, 'filepage', 4);
			}
			
			if ($make_box) {
				$msg_top=
					t3lib_div::makeInstance(
						't3lib_FlashMessage',
						$security_msg_body,
						$security_msg_title,
						$box_type
					)->render();
			} else {
				$msg_bottom= '<p style="margin-top: 20px; margin-bottom: 20px">'.(strlen($security_msg_title) > 0 ? '<b>'.$security_msg_title.': </b>' : '').$security_msg_body.'</p>';
			}
		}

		$params= '&edit[pages]['.$id.']=edit';
		$editlink= 
			'<a href="#" onclick="' . htmlspecialchars(t3lib_BEfunc::editOnClick($params, $backPath)) . '" title="bearbeiten">' .
			(
				method_exists('t3lib_iconWorks', 'getSpriteIcon')
				?
				t3lib_iconWorks::getSpriteIcon('actions-page-open')
				:
				'<img ' . t3lib_iconWorks::skinImg($backPath, 'gfx/edit2.gif', 'width="11" height="12"') . ' border="0" alt="">'
			).
			'</a>';
		
		$filenames= $pageRecord['tx_filepage_upload'];
		
		if (strlen($filenames) == 0) {
			$flashMessage = 
				t3lib_div::makeInstance(
					't3lib_FlashMessage',
					sprintf($GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:error.file_name_missing_text'), $editlink),
					$GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:error.file_name_missing'),
					t3lib_FlashMessage::ERROR
				);
			return $msg_top.$flashMessage->render().$msg_bottom;
		}
		
		$filename_array= explode(',', $filenames); // get file names as array
		$active_filename= array_pop($filename_array); // the last file name in the list is the active file
		$fileabs= PATH_site.tx_filepage_config::get('uploadfolder').'/'.$active_filename;
		
		if (!file_exists($fileabs)) {
			$flashMessage = 
				t3lib_div::makeInstance(
					't3lib_FlashMessage',
					$GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:error.file_not_exists_text').'<br />file name: '.$active_filename,
					$GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:error.file_not_exists'),
					t3lib_FlashMessage::ERROR
				);
			return $msg_top.$flashMessage->render().$msg_bottom;
		}
		
		if (!is_readable($fileabs)) {
			$flashMessage = 
				t3lib_div::makeInstance(
					't3lib_FlashMessage',
					$GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:error.file_not_readable_text').'<br />file name: '.$active_filename,
					$GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:error.file_not_readable'),
					t3lib_FlashMessage::ERROR
				);
			return $msg_top.$flashMessage->render().$msg_bottom;
		}
		
		$linktext= $active_filename;
		$url= t3lib_div::getIndpEnv('TYPO3_SITE_URL').'?'.(strlen($pageRecord['alias']) > 0 ? $pageRecord['alias'] : $id);
		$previewlink= '<a target="_blank" href="'.$url.'">'.$linktext.'</a>';

		$patchedRecord= $pageRecord;
		$patchedRecord['tx_filepage_upload']= $active_filename; // show only the active file
		$img= t3lib_BEfunc::thumbCode($patchedRecord, 'pages', 'tx_filepage_upload', $backPath);

		$filesize= t3lib_div::formatSize(filesize($fileabs)).'bytes';
		
		$flashMessage = 
			t3lib_div::makeInstance(
				't3lib_FlashMessage',
				sprintf($GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:info.file_text'), $previewlink, $editlink, $img, $filesize),
				$GLOBALS['LANG']->sL('LLL:EXT:filepage/locallang.xml:info.file'),
				t3lib_FlashMessage::INFO
			);
		
		return $msg_top.$flashMessage->render().$msg_bottom;		
	}
	
}

// this is a static class. therefore we have no xclass clause here. 

?>